@@ -20,6 +20,10 @@ def darwin?
   RbConfig::CONFIG['target_os'] =~ /darwin/
 end
 
+def openbsd?
+  RbConfig::CONFIG['target_os'] =~ /openbsd/
+end
+
 def nix?
   ! (windows? || solaris? || darwin?)
 end
@@ -62,7 +66,7 @@ usage: ruby #{$0} [options]
         Use the zlib library placed under DIR.
 
     --use-system-libraries
-        Use system libraries intead of building and using the bundled
+        Use system libraries instead of building and using the bundled
         libraries.
 
     --with-xml2-dir=DIR / --with-xml2-config=CONFIG
@@ -112,7 +116,7 @@ def package_config pkg, options={}
 
   begin
     require 'rubygems'
-    gem 'pkg-config', (gem_ver='~> 1.1.7')
+    gem 'pkg-config', (gem_ver='~> 1.1')
     require 'pkg-config' and message("Using pkg-config gem version #{PKGConfig::VERSION}\n")
   rescue LoadError
     message "pkg-config could not be used to find #{pkg}\nPlease install either `pkg-config` or the pkg-config gem per\n\n    gem install pkg-config -v #{gem_ver.inspect}\n\n"
@@ -131,12 +135,7 @@ def package_config pkg, options={}
 end
 
 def nokogiri_try_compile
-  args = if defined?(RUBY_VERSION) && RUBY_VERSION <= "1.9.2"
-           ["int main() {return 0;}"]
-         else
-           ["int main() {return 0;}", "", {werror: true}]
-         end
-  try_compile(*args)
+  try_compile "int main() {return 0;}", "", {werror: true}
 end
 
 def check_libxml_version version=nil
@@ -224,8 +223,8 @@ def iconv_configure_flags
 
       return [
         '--with-iconv=yes',
-        *("CPPFLAGS=#{idirs.map { |dir| '-I' << dir }.join(' ')}" if idirs),
-        *("LDFLAGS=#{ldirs.map { |dir| '-L' << dir }.join(' ')}" if ldirs),
+        *("CPPFLAGS=#{idirs.map { |dir| '-I' + dir }.join(' ')}" if idirs),
+        *("LDFLAGS=#{ldirs.map { |dir| '-L' + dir }.join(' ')}" if ldirs),
       ]
     end
   end
@@ -261,11 +260,12 @@ end
 
 def process_recipe(name, version, static_p, cross_p)
   MiniPortile.new(name, version).tap do |recipe|
-    recipe.target = portsdir = File.join(ROOT, "ports")
+    recipe.target = File.join(ROOT, "ports")
     # Prefer host_alias over host in order to use i586-mingw32msvc as
     # correct compiler prefix for cross build, but use host if not set.
     recipe.host = RbConfig::CONFIG["host_alias"].empty? ? RbConfig::CONFIG["host"] : RbConfig::CONFIG["host_alias"]
     recipe.patch_files = Dir[File.join(ROOT, "patches", name, "*.patch")].sort
+    recipe.configure_options << "--libdir=#{File.join(recipe.path, "lib")}"
 
     yield recipe
 
@@ -308,7 +308,7 @@ def process_recipe(name, version, static_p, cross_p)
     if RbConfig::CONFIG['target_cpu'] == 'universal'
       %w[CFLAGS LDFLAGS].each do |key|
         unless env[key].include?('-arch')
-          env[key] << ' ' << RbConfig::CONFIG['ARCH_FLAG']
+          env[key] += ' ' + RbConfig::CONFIG['ARCH_FLAG']
         end
       end
     end
@@ -381,14 +381,6 @@ def using_system_libraries?
   arg_config('--use-system-libraries', !!ENV['NOKOGIRI_USE_SYSTEM_LIBRARIES'])
 end
 
-#
-# monkey patches
-#
-
-# Workaround for Ruby bug #8074, introduced in Ruby 2.0.0, fixed in Ruby 2.1.0
-# https://bugs.ruby-lang.org/issues/8074
-@libdir_basename = "lib" if RUBY_VERSION < '2.1.0'
-
 #
 # main
 #
@@ -400,15 +392,18 @@ when arg_config('--clean')
   do_clean
 end
 
+if openbsd? && !using_system_libraries?
+  if `#{ENV['CC'] || '/usr/bin/cc'} -v 2>&1` !~ /clang/
+    ENV['CC'] ||= find_executable('egcc') or
+      abort "Please install gcc 4.9+ from ports using `pkg_add -v gcc`"
+  end
+  ENV['CFLAGS'] = "#{ENV['CFLAGS']} -I /usr/local/include"
+end
+
 RbConfig::MAKEFILE_CONFIG['CC'] = ENV['CC'] if ENV['CC']
 # use same c compiler for libxml and libxslt
 ENV['CC'] = RbConfig::MAKEFILE_CONFIG['CC']
 
-# TODO: deprecate MacRuby: https://github.com/sparklemotion/nokogiri/issues/1474
-if defined?(RUBY_ENGINE) && RUBY_ENGINE == 'macruby'
-  $LIBRUBYARG_STATIC.gsub!(/-static/, '')
-end
-
 $LIBS << " #{ENV["LIBS"]}"
 
 # Read CFLAGS from ENV and make sure compiling works.
@@ -465,7 +460,7 @@ else
   # The gem version constraint in the Rakefile is not respected at install time.
   # Keep this version in sync with the one in the Rakefile !
   require 'rubygems'
-  gem 'mini_portile2', '~> 2.1.0'
+  gem 'mini_portile2', '~> 2.3.0'
   require 'mini_portile2'
   message "Using mini_portile version #{MiniPortile::VERSION}\n"
 
@@ -482,8 +477,8 @@ else
   if cross_build_p || windows?
     zlib_recipe = process_recipe("zlib", dependencies["zlib"]["version"], static_p, cross_build_p) do |recipe|
       recipe.files = [{
-          url: "http://zlib.net/#{recipe.name}-#{recipe.version}.tar.gz",
-          md5: dependencies["zlib"]["md5"]
+          url: "http://zlib.net/fossils/#{recipe.name}-#{recipe.version}.tar.gz",
+          sha256: dependencies["zlib"]["sha256"]
         }]
       class << recipe
         attr_accessor :cross_build_p
@@ -521,7 +516,7 @@ else
     libiconv_recipe = process_recipe("libiconv", dependencies["libiconv"]["version"], static_p, cross_build_p) do |recipe|
       recipe.files = [{
           url: "http://ftp.gnu.org/pub/gnu/libiconv/#{recipe.name}-#{recipe.version}.tar.gz",
-          md5: dependencies["libiconv"]["md5"]
+          sha256: dependencies["libiconv"]["sha256"]
         }]
       recipe.configure_options += [
         "CPPFLAGS=-Wall",
@@ -556,7 +551,7 @@ EOM
   libxml2_recipe = process_recipe("libxml2", dependencies["libxml2"]["version"], static_p, cross_build_p) do |recipe|
     recipe.files = [{
         url: "http://xmlsoft.org/sources/#{recipe.name}-#{recipe.version}.tar.gz",
-        md5: dependencies["libxml2"]["md5"]
+        sha256: dependencies["libxml2"]["sha256"]
       }]
     recipe.configure_options += [
       "--without-python",
@@ -572,7 +567,7 @@ EOM
   libxslt_recipe = process_recipe("libxslt", dependencies["libxslt"]["version"], static_p, cross_build_p) do |recipe|
     recipe.files = [{
         url: "http://xmlsoft.org/sources/#{recipe.name}-#{recipe.version}.tar.gz",
-        md5: dependencies["libxslt"]["md5"]
+        sha256: dependencies["libxslt"]["sha256"]
       }]
     recipe.configure_options += [
       "--without-python",
@@ -666,12 +661,6 @@ have_func('xmlRelaxNGSetValidStructuredErrors')
 have_func('xmlSchemaSetValidStructuredErrors')
 have_func('xmlSchemaSetParserStructuredErrors')
 
-if ENV['CPUPROFILE']
-  unless find_library('profiler', 'ProfilerEnable', *LIB_DIRS)
-    abort "google performance tools are not installed"
-  end
-end
-
 create_makefile('nokogiri/nokogiri')
 
 if enable_config('clean', true)
