@@ -1,3 +1,4 @@
+# encoding: UTF-8
 require 'stringio'
 require 'nokogiri/xml/node/save_options'
 
@@ -37,7 +38,7 @@ module Nokogiri
     # its subtree), there are a few methods you might want to use:
     #
     # * content, text, inner_text, to_str: emit plaintext
-    #  
+    #
     #   These methods will all emit the plaintext version of your
     #   document, meaning that entities will be replaced (e.g., "&lt;"
     #   will be replaced with "<"), meaning that any sanitizing will
@@ -153,7 +154,7 @@ module Nokogiri
       def prepend_child node_or_tags
         if first = children.first
           # Mimic the error add_child would raise.
-          raise RuntimeError, "Document already has a root node" if document? && !node_or_tags.processing_instruction?
+          raise RuntimeError, "Document already has a root node" if document? && !(node_or_tags.comment? || node_or_tags.processing_instruction?)
           first.__send__(:add_sibling, :previous, node_or_tags)
         else
           add_child(node_or_tags)
@@ -171,6 +172,7 @@ module Nokogiri
         add_child node_or_tags
         self
       end
+
       ###
       # Insert +node_or_tags+ before this Node (as a sibling).
       # +node_or_tags+ can be a Nokogiri::XML::Node, a ::DocumentFragment, a ::NodeSet, or a string containing markup.
@@ -179,7 +181,7 @@ module Nokogiri
       #
       # Also see related method +before+.
       def add_previous_sibling node_or_tags
-        raise ArgumentError.new("A document may not have multiple root nodes.") if (parent && parent.document?) && !node_or_tags.processing_instruction?
+        raise ArgumentError.new("A document may not have multiple root nodes.") if (parent && parent.document?) && !(node_or_tags.comment? || node_or_tags.processing_instruction?)
 
         add_sibling :previous, node_or_tags
       end
@@ -192,7 +194,7 @@ module Nokogiri
       #
       # Also see related method +after+.
       def add_next_sibling node_or_tags
-        raise ArgumentError.new("A document may not have multiple root nodes.") if (parent && parent.document?) && !node_or_tags.processing_instruction?
+        raise ArgumentError.new("A document may not have multiple root nodes.") if (parent && parent.document?) && !(node_or_tags.comment? || node_or_tags.processing_instruction?)
 
         add_sibling :next, node_or_tags
       end
@@ -331,13 +333,13 @@ module Nokogiri
       ###
       # Get the attribute values for this Node.
       def values
-        attribute_nodes.map { |node| node.value }
+        attribute_nodes.map(&:value)
       end
 
       ###
       # Get the attribute names for this Node.
       def keys
-        attribute_nodes.map { |node| node.node_name }
+        attribute_nodes.map(&:node_name)
       end
 
       ###
@@ -386,7 +388,7 @@ module Nokogiri
         end
 
         options ||= (document.html? ? ParseOptions::DEFAULT_HTML : ParseOptions::DEFAULT_XML)
-        if Fixnum === options
+        if Integer === options
           options = Nokogiri::XML::ParseOptions.new(options)
         end
         # Give the options to the user
@@ -439,12 +441,6 @@ module Nokogiri
       def namespaces
         Hash[namespace_scopes.map { |nd|
           key = ['xmlns', nd.prefix].compact.join(':')
-          if RUBY_VERSION >= '1.9' && document.encoding
-            begin
-              key.force_encoding document.encoding
-            rescue ArgumentError
-            end
-          end
           [key, nd.href]
         }]
       end
@@ -546,9 +542,10 @@ module Nokogiri
         return NodeSet.new(document, parents) unless selector
 
         root = parents.last
+        search_results = root.search(selector)
 
         NodeSet.new(document, parents.find_all { |parent|
-          root.search(selector).include?(parent)
+          search_results.include?(parent)
         })
       end
 
@@ -626,10 +623,8 @@ module Nokogiri
         encoding = options[:encoding] || document.encoding
         options[:encoding] = encoding
 
-        outstring = ""
-        if encoding && outstring.respond_to?(:force_encoding)
-          outstring.force_encoding(Encoding.find(encoding))
-        end
+        outstring = String.new
+        outstring.force_encoding(Encoding.find(encoding || 'utf-8'))
         io = StringIO.new(outstring)
         write_to io, options, &block
         io.string
@@ -682,7 +677,7 @@ module Nokogiri
       #
       # To save indented with two dashes:
       #
-      #   node.write_to(io, :indent_text => '-', :indent => 2
+      #   node.write_to(io, :indent_text => '-', :indent => 2)
       #
       def write_to io, *options
         options       = options.first.is_a?(Hash) ? options.shift : {}
@@ -743,7 +738,7 @@ module Nokogiri
       # Nokogiri::XML::ParseOptions object initialized from +options+, will be
       # passed to it, allowing more convenient modification of the parser options.
       def do_xinclude options = XML::ParseOptions::DEFAULT_XML, &block
-        options = Nokogiri::XML::ParseOptions.new(options) if Fixnum === options
+        options = Nokogiri::XML::ParseOptions.new(options) if Integer === options
 
         # give options to user
         yield options if block_given?
@@ -822,9 +817,8 @@ Requires a Node, NodeSet or String argument, and cannot accept a #{data.class}.
         EOERR
       end
 
-      def implied_xpath_contexts # :nodoc:
-        [".//"]
-      end
+      # @private
+      IMPLIED_XPATH_CONTEXTS = [ './/'.freeze ].freeze # :nodoc:
 
       def add_child_node_and_reparent_attrs node # :nodoc:
         add_child_node node
